@@ -1,4 +1,4 @@
-# Copyright (c) 2021, 2022, Panagiotis Tsirigotis
+# Copyright (c) 2021, 2022, 2023, Panagiotis Tsirigotis
 
 # This file is part of linuxnet-iptables.
 #
@@ -25,7 +25,7 @@ from typing import Callable, List, Optional
 from .exceptions import (
         IptablesError, IptablesParsingError, IptablesExecutionError)
 from .rule import ChainRule
-from .target import ChainTarget
+from .targets import ChainTarget
 from .deps import get_logger
 
 _logger = get_logger('linuxnet.iptables.chain')
@@ -214,8 +214,8 @@ class Chain:
 
         :param match: optional :class:`Match` object; use a :class:`MatchNone`
             object to find a rule that has no matches
-        :param target: optional :class:`Target` object; use a :class:`TargetNone`
-            object to find a rule that has no target
+        :param target: optional :class:`Target` object; use a
+            :class:`TargetNone` object to find a rule that has no target
         """
         if match is None and target is None:
             return []
