@@ -1,4 +1,4 @@
-# Copyright (c) 2021, 2022, Panagiotis Tsirigotis
+# Copyright (c) 2021, 2022, 2023, Panagiotis Tsirigotis
 
 # This file is part of linuxnet-iptables.
 #
@@ -27,8 +27,8 @@ from typing import Dict, Iterator, List, Optional, Tuple
 from .exceptions import (
                 IptablesError, IptablesParsingError, IptablesExecutionError)
 from .chain import Chain
-from .match import CommentMatch
-from .target import Target, ChainTarget, UnparsedTarget
+from .matches.commentmatch import CommentMatch
+from .targets import Target, ChainTarget, UnparsedTarget
 from .deps import get_logger
 
 _logger = get_logger("linuxnet.iptables.table")
