@@ -34,14 +34,24 @@ else:
     sys.path.insert(0, '.')
     TESTDIR = 'tests'
 
-from linuxnet.iptables import IptablesPacketFilterTable
-from linuxnet.iptables import IptablesError
-from linuxnet.iptables import ChainRule, ChainTarget, Targets, MarkTarget
-from linuxnet.iptables import CommentMatch
-from linuxnet.iptables import TcpmssMatch, TcpMatch
-from linuxnet.iptables import UdpMatch, IcmpMatch
-from linuxnet.iptables import PacketTypeMatch
-from linuxnet.iptables import ConnmarkMatch
+from linuxnet.iptables import (
+                IptablesPacketFilterTable,
+                IptablesError,
+                ChainRule,
+                # Targets
+                ChainTarget, Targets,
+                MarkTarget, ConnmarkTarget,
+                RejectTarget,
+                MasqueradeTarget,
+                TtlTarget,
+                # Matches
+                CommentMatch,
+                TcpmssMatch, TcpMatch,
+                UdpMatch, IcmpMatch,
+                PacketTypeMatch,
+                ConnmarkMatch,
+                LimitMatch,
+                )
 
 root_logger = logging.getLogger()
 root_logger.addHandler(logging.FileHandler('test.log', 'w'))
@@ -251,7 +261,7 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
                 self.assertTrue(isinstance(match, TcpmssMatch))
                 mss = match.mss()
                 self.assertTrue(mss.is_positive() and
-                                        mss.get_value() == '1:500')
+                                        mss.get_value() == (1, 500))
             elif rulenum == 2:
                 self.assertTrue(isinstance(match, TcpMatch))
                 flags = match.tcp_flags()
@@ -293,12 +303,18 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
 Chain INPUT (policy DROP 0 packets, 0 bytes)
     pkts      bytes target     prot opt in     out     source               destination
   143588 11622518 DROP         icmp --  *      *       0.0.0.0/0            0.0.0.0/0           icmp type 8
+  143588 11622518 DROP         icmp --  *      *       0.0.0.0/0            0.0.0.0/0           icmp !any
+  143588 11622518 DROP         icmp --  *      *       0.0.0.0/0            0.0.0.0/0           icmp type 3 icmp !type 3 code 1
+  143588 11622518 DROP         icmp --  *      *       0.0.0.0/0            0.0.0.0/0           icmptype 3
 """ + '\n' + self.EMPTY_FORWARD + '\n' + self.EMPTY_OUTPUT
         pft = IptablesPacketFilterTable('filter')
         self.assertTrue(pft.init_from_output(output))
         input_chain = pft.get_builtin_chain('INPUT')
         rules = input_chain.get_rules()
-        self.assertEqual(len(rules), 1)
+        self.assertEqual(len(rules), 4)
+        #
+        # Verify first rule
+        #
         rule = rules[0]
         match_list = rule.get_match_list()
         # There is a PacketMatch matching prot
@@ -307,7 +323,47 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         self.assertTrue(isinstance(match, IcmpMatch))
         icmp_type = match.icmp_type()
         self.assertTrue(icmp_type.is_positive() and
-                        icmp_type.get_value()[0] == 'echo-request')
+                        icmp_type.get_type_name() == 'echo-request' and
+                        icmp_type.get_type_value() == 8)
+        #
+        # Verify second rule
+        #
+        rule = rules[1]
+        match_list = rule.get_match_list()
+        self.assertEqual(len(match_list), 2)
+        match = match_list[-1]
+        self.assertTrue(isinstance(match, IcmpMatch))
+        icmp_type = match.icmp_type()
+        self.assertTrue(not icmp_type.is_positive() and
+                        icmp_type.get_type_name() == 'any')
+        #
+        # Verify third rule
+        #
+        rule = rules[2]
+        match_list = rule.get_match_list()
+        self.assertEqual(len(match_list), 3)
+        match = match_list[-2]
+        self.assertTrue(isinstance(match, IcmpMatch))
+        icmp_type = match.icmp_type()
+        self.assertTrue(icmp_type.is_positive() and
+                        icmp_type.get_type_value() == 3)
+        match = match_list[-1]
+        self.assertTrue(isinstance(match, IcmpMatch))
+        icmp_type = match.icmp_type()
+        self.assertTrue(not icmp_type.is_positive() and
+                        icmp_type.get_type_value() == 3 and
+                        icmp_type.get_code() == 1)
+        #
+        # Verify fourth rule
+        #
+        rule = rules[3]
+        match_list = rule.get_match_list()
+        self.assertEqual(len(match_list), 2)
+        match = match_list[-1]
+        self.assertTrue(isinstance(match, IcmpMatch))
+        icmp_type = match.icmp_type()
+        self.assertTrue(icmp_type.is_positive() and
+                        icmp_type.get_type_value() == 3)
 
     def test_parsing_connmark_match(self):
         """Parse output with connmark match
@@ -349,7 +405,7 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         match = match_list[0]
         limit = match.limit()
         self.assertTrue(limit.is_positive() and
-                limit.get_value() == 21600)       # 21600/day
+                limit.get_value() == LimitMatch.Rate(15, LimitMatch.Rate.PER_MIN))
         burst = match.burst()
         self.assertTrue(burst.is_positive() and
                 burst.get_value() == 5)
@@ -437,7 +493,7 @@ Chain INPUT (policy ACCEPT 28770515 packets, 54867569862 bytes)
         output = """\
 Chain INPUT (policy DROP 0 packets, 0 bytes)
     pkts      bytes target     prot opt in     out     source               destination
-  487148 21609270 LOG        all  --  *      *       0.0.0.0/0            0.0.0.0/0           limit: avg 15/min burst 5 LOG flags 0 level 6 prefix `DROP-INPUT: '
+  487148 21609270 LOG        all  --  *      *       0.0.0.0/0            0.0.0.0/0           limit: avg 15/min burst 5 LOG flags 12 level 6 prefix `DROP-INPUT: '
   520139 23069461 DROP       all  --  *      *       0.0.0.0/0            0.0.0.0/0
 """ + '\n' + self.EMPTY_FORWARD + '\n' + self.EMPTY_OUTPUT
         pft = IptablesPacketFilterTable('filter')
@@ -447,7 +503,10 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         target = rule.get_target()
         self.assertEqual(target.get_log_prefix(), 'DROP-INPUT: ')
         self.assertEqual(target.get_log_level(), '6')
-        self.assertEqual(target.get_log_options(), [])
+        self.assertTrue(target.is_logging_uid())
+        self.assertTrue(target.is_logging_ip_options())
+        self.assertFalse(target.is_logging_tcp_options())
+        self.assertFalse(target.is_logging_tcp_sequence())
 
     def test_parsing_reject_target(self):
         """Parse output with REJECT target
@@ -463,7 +522,7 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         input_chain = pft.get_builtin_chain('INPUT')
         rule = input_chain.get_rules()[0]
         target = rule.get_target()
-        self.assertEqual(target, Targets.REJECT)
+        self.assertTrue(isinstance(target, RejectTarget))
         self.assertEqual(target.get_rejection_message(),
                                 'icmp-host-unreachable')
 
@@ -485,32 +544,32 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         # Verify SET
         rule = input_chain.get_rules()[0]
         target = rule.get_target()
-        self.assertEqual(target, Targets.MARK)
+        self.assertTrue(isinstance(target, MarkTarget))
         self.assertEqual(target.get_op(), MarkTarget.SET)
         self.assertEqual(target.get_mark(), 0xf)
         # Verify XSET
         rule = input_chain.get_rules()[1]
         target = rule.get_target()
-        self.assertEqual(target, Targets.MARK)
+        self.assertTrue(isinstance(target, MarkTarget))
         self.assertEqual(target.get_op(), MarkTarget.XSET)
         self.assertEqual(target.get_mark(), 0x11)
         self.assertEqual(target.get_mask(), 0xffff0011)
         # Verify XOR
         rule = input_chain.get_rules()[2]
         target = rule.get_target()
-        self.assertEqual(target, Targets.MARK)
+        self.assertTrue(isinstance(target, MarkTarget))
         self.assertEqual(target.get_op(), MarkTarget.XOR)
         self.assertEqual(target.get_mask(), 0xf)
         # Verify OR
         rule = input_chain.get_rules()[3]
         target = rule.get_target()
-        self.assertEqual(target, Targets.MARK)
+        self.assertTrue(isinstance(target, MarkTarget))
         self.assertEqual(target.get_op(), MarkTarget.OR)
         self.assertEqual(target.get_mask(), 0xf)
         # Verify AND
         rule = input_chain.get_rules()[4]
         target = rule.get_target()
-        self.assertEqual(target, Targets.MARK)
+        self.assertTrue(isinstance(target, MarkTarget))
         self.assertEqual(target.get_op(), MarkTarget.AND)
         self.assertEqual(target.get_mask(), 0xff)
 
@@ -521,6 +580,8 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
 Chain PREROUTING (policy DROP 0 packets, 0 bytes)
     pkts      bytes target     prot opt in     out     source               destination
   558864 28503489 CONNMARK   all  --  *      *       0.0.0.0/0            0.0.0.0/0           connmark match 0x0 CONNMARK set 0x11 
+  558864 28503489 CONNMARK   all  --  *      *       0.0.0.0/0            0.0.0.0/0           connmark match 0x0 CONNMARK save nfmask 0xfffff ctmask ~0x1f
+  558864 28503489 CONNMARK   all  --  *      *       0.0.0.0/0            0.0.0.0/0           connmark match 0x0 CONNMARK restore ctmask 0x1f nfmask ~0xfffff
 """ + '\n' + self.EMPTY_FORWARD + '\n' + self.EMPTY_OUTPUT + '\n' +
                 self.EMPTY_INPUT + '\n' + self.EMPTY_POSTROUTING)
         pft = IptablesPacketFilterTable('mangle')
@@ -528,8 +589,47 @@ Chain PREROUTING (policy DROP 0 packets, 0 bytes)
         prerouting_chain = pft.get_builtin_chain('PREROUTING')
         rule = prerouting_chain.get_rules()[0]
         target = rule.get_target()
-        self.assertEqual(target, Targets.CONNMARK)
+        self.assertTrue(isinstance(target, ConnmarkTarget))
         self.assertEqual(target.get_mark(), 0x11)
+        rule = prerouting_chain.get_rules()[1]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, ConnmarkTarget) and
+                                target.is_saving_mark())
+        self.assertEqual(target.get_nfmask(), 0xfffff)
+        self.assertEqual(target.get_ctmask(), 0x1f)
+        rule = prerouting_chain.get_rules()[2]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, ConnmarkTarget) and
+                                target.is_restoring_mark())
+        self.assertEqual(target.get_nfmask(), 0xfffff)
+        self.assertEqual(target.get_ctmask(), 0x1f)
+
+    def test_parsing_ttl_target(self):
+        """Parse output with TTL target
+        """
+        output = ("""\
+Chain PREROUTING (policy DROP 0 packets, 0 bytes)
+    pkts      bytes target     prot opt in     out     source               destination
+       0        0 TTL        all  --  *      *       0.0.0.0/0            0.0.0.0/0           TTL set to 10 
+       0        0 TTL        tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           TTL increment by 1 
+       0        0 TTL        udp  --  *      *       0.0.0.0/0            0.0.0.0/0           TTL decrement by 2 
+""" + '\n' + self.EMPTY_FORWARD + '\n' + self.EMPTY_OUTPUT + '\n' +
+                self.EMPTY_INPUT + '\n' + self.EMPTY_POSTROUTING)
+        pft = IptablesPacketFilterTable('mangle')
+        self.assertTrue(pft.init_from_output(output))
+        prerouting_chain = pft.get_builtin_chain('PREROUTING')
+        rule = prerouting_chain.get_rules()[0]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, TtlTarget) and
+                                target.get_ttl_value() == 10)
+        rule = prerouting_chain.get_rules()[1]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, TtlTarget) and
+                                target.get_ttl_inc() == 1)
+        rule = prerouting_chain.get_rules()[2]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, TtlTarget) and
+                                target.get_ttl_dec() == 2)
 
     def test_parsing_snat_target(self):
         """Parse output with SNAT target
@@ -547,6 +647,42 @@ Chain POSTROUTING (1 references)
         self.assertEqual(target.get_target_name(), 'SNAT')
         self.assertEqual(target.get_address(), IPv4Address('10.10.10.18'))
 
+    def test_parsing_masquerade_target(self):
+        """Parse output with MASQUERADE target
+        """
+        output = (self.EMPTY_PREROUTING + '\n' + """\
+Chain POSTROUTING (1 references)
+    pkts      bytes target     prot opt in     out     source               destination         
+       0        0 MASQUERADE  all  --  *      *       0.0.0.0/0            0.0.0.0/0           
+       0        0 MASQUERADE  all  --  *      *       0.0.0.0/0            0.0.0.0/0           random 
+       0        0 MASQUERADE  tcp  --  *      *       0.0.0.0/0            0.0.0.0/0           masq ports: 2000 
+       0        0 MASQUERADE  udp  --  *      *       0.0.0.0/0            0.0.0.0/0           udp spts:1000:2000 masq ports: 2000 random 
+       0        0 MASQUERADE  udp  --  *      *       0.0.0.0/0            0.0.0.0/0           udp spts:1000:2000 masq ports: 20000-30000
+""" + '\n' + self.EMPTY_OUTPUT)
+        pft = IptablesPacketFilterTable('nat')
+        self.assertTrue(pft.init_from_output(output))
+        postrouting_chain = pft.get_builtin_chain('POSTROUTING')
+        rule = postrouting_chain.get_rules()[0]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, MasqueradeTarget))
+        rule = postrouting_chain.get_rules()[1]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, MasqueradeTarget) and
+                                target.uses_random_port_mapping())
+        rule = postrouting_chain.get_rules()[2]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, MasqueradeTarget) and
+                                target.get_ports() == (2000, None))
+        rule = postrouting_chain.get_rules()[3]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, MasqueradeTarget) and
+                            target.get_ports() == (2000, None) and
+                                target.uses_random_port_mapping())
+        rule = postrouting_chain.get_rules()[4]
+        target = rule.get_target()
+        self.assertTrue(isinstance(target, MasqueradeTarget) and
+                                target.get_ports() == (20000, 30000))
+
     def test_parsing_unknown_target(self):
         """Parse output with unknown target
         """
